@@ -1,5 +1,6 @@
 # pep8: noqa
 from .core import *
+from .util import clear_neo4j_database, change_neo4j_password
 from .relationship_manager import (NotConnected, OUTGOING, INCOMING, EITHER,
         RelationshipTo, RelationshipFrom, Relationship, RelationshipManager, RelationshipDefinition)
 from .relationship import StructuredRel
@@ -7,12 +8,12 @@ from .cardinality import (AttemptedCardinalityViolation,
         CardinalityViolation, ZeroOrMore, OneOrMore, ZeroOrOne, One)
 from .properties import (StringProperty, IntegerProperty, AliasProperty,
         FloatProperty, BooleanProperty, DateTimeProperty, DateProperty,
-        JSONProperty, ArrayProperty)
-from .exception import InflateError, DeflateError, UniqueProperty, CypherException, MultipleNodesReturned
-from .signals import SIGNAL_SUPPORT
+        NormalProperty, RegexProperty, EmailProperty,
+        JSONProperty, ArrayProperty, UniqueIdProperty)
+from .exception import InflateError, DeflateError, UniqueProperty, MultipleNodesReturned, RequiredProperty
 
 __author__ = 'Robin Edwards'
 __email__ = 'robin.ge@gmail.com'
 __license__ = 'MIT'
 __package__ = 'neomodel'
-__version__ = '1.0.2'
+__version__ = '3.2.4'
