@@ -2,33 +2,113 @@ from .core import StructuredNode, db
 from .properties import AliasProperty
 from .exception import MultipleNodesReturned
 import inspect
+import re
 OUTGOING, INCOMING, EITHER = 1, -1, 0
 
+# basestring python 3.x fallback
+try:
+    basestring
+except NameError:
+    basestring = str
 
-def rel_helper(**rel):
-    if rel['direction'] == OUTGOING:
-        stmt = '-[{0}:{1}]->'
-    elif rel['direction'] == INCOMING:
-        stmt = '<-[{0}:{1}]-'
+
+def _rel_helper(lhs, rhs, ident=None, relation_type=None, direction=None, relation_properties=None, **kwargs):
+    """
+    Generate a relationship matching string, with specified parameters.
+    Examples:
+    relation_direction = OUTGOING: (lhs)-[relation_ident:relation_type]->(rhs)
+    relation_direction = INCOMING: (lhs)<-[relation_ident:relation_type]-(rhs)
+    relation_direction = EITHER: (lhs)-[relation_ident:relation_type]-(rhs)
+
+    :param lhs: The left hand statement.
+    :type lhs: str
+    :param rhs: The right hand statement.
+    :type rhs: str
+    :param ident: A specific identity to name the relationship, or None.
+    :type ident: str
+    :param relation_type: None for all direct rels, * for all of any length, or a name of an explicit rel.
+    :type relation_type: str
+    :param direction: None or EITHER for all OUTGOING,INCOMING,EITHER. Otherwise OUTGOING or INCOMING.
+    :param relation_properties: dictionary of relationship properties to match
+    :returns: string
+    """
+
+    if direction == OUTGOING:
+        stmt = '-{0}->'
+    elif direction == INCOMING:
+        stmt = '<-{0}-'
     else:
-        stmt = '-[{0}:{1}]-'
-    ident = rel['ident'] if 'ident' in rel else ''
-    stmt = stmt.format(ident, rel['relation_type'])
-    return "({0}){1}({2})".format(rel['lhs'], stmt, rel['rhs'])
+        stmt = '-{0}-'
+
+    rel_props = ''
+
+    if relation_properties:
+        rel_props = ' {{{0}}}'.format(', '.join(
+            ['{}: {}'.format(key, value) for key, value in relation_properties.items()]))
+
+    # direct, relation_type=None is unspecified, relation_type
+    if relation_type is None:
+        stmt = stmt.format('')
+    # all("*" wildcard) relation_type
+    elif relation_type == '*':
+        stmt = stmt.format('[*]')
+    else:
+        # explicit relation_type
+        stmt = stmt.format('[%s:`%s`%s]' % (ident if ident else '', relation_type, rel_props))
+
+    return "({0}){1}({2})".format(lhs, stmt, rhs)
 
 
+# special operators
+_SPECIAL_OPERATOR_IN = 'IN'
+_SPECIAL_OPERATOR_INSENSITIVE = '(?i)'
+_SPECIAL_OPERATOR_ISNULL = 'IS NULL'
+_SPECIAL_OPERATOR_ISNOTNULL = 'IS NOT NULL'
+_SPECIAL_OPERATOR_REGEX = '=~'
+
+_UNARY_OPERATORS = (_SPECIAL_OPERATOR_ISNULL, _SPECIAL_OPERATOR_ISNOTNULL)
+
+_REGEX_INSESITIVE = _SPECIAL_OPERATOR_INSENSITIVE + '{}'
+_REGEX_CONTAINS = '.*{}.*'
+_REGEX_STARTSWITH = '{}.*'
+_REGEX_ENDSWITH = '.*{}'
+
+# regex operations that require escaping
+_STRING_REGEX_OPERATOR_TABLE = {
+    'iexact': _REGEX_INSESITIVE,
+    'contains': _REGEX_CONTAINS,
+    'icontains': _SPECIAL_OPERATOR_INSENSITIVE + _REGEX_CONTAINS,
+    'startswith': _REGEX_STARTSWITH,
+    'istartswith': _SPECIAL_OPERATOR_INSENSITIVE + _REGEX_STARTSWITH,
+    'endswith': _REGEX_ENDSWITH,
+    'iendswith': _SPECIAL_OPERATOR_INSENSITIVE + _REGEX_ENDSWITH,
+}
+# regex operations that do not require escaping
+_REGEX_OPERATOR_TABLE = {
+    'iregex': _REGEX_INSESITIVE,
+}
+# list all regex operations, these will require formatting of the value
+_REGEX_OPERATOR_TABLE.update(_STRING_REGEX_OPERATOR_TABLE)
+
+# list all supported operators
 OPERATOR_TABLE = {
     'lt': '<',
     'gt': '>',
     'lte': '<=',
     'gte': '>=',
     'ne': '<>',
+    'in': _SPECIAL_OPERATOR_IN,
+    'isnull': _SPECIAL_OPERATOR_ISNULL,
+    'regex': _SPECIAL_OPERATOR_REGEX,
+    'exact': '='
 }
+# add all regex operators
+OPERATOR_TABLE.update(_REGEX_OPERATOR_TABLE)
 
 
 def install_traversals(cls, node_set):
     """
-    from a StructuredNode class install Traversal objects for each
+    For a StructuredNode class install Traversal objects for each
     relationship definition on a NodeSet instance
     """
     rels = cls.defined_properties(rels=True, aliases=False, properties=False)
@@ -68,9 +148,31 @@ def process_filter_args(cls, kwargs):
             prop = property_obj.aliased_to()
             deflated_value = getattr(cls, prop).deflate(value)
         else:
-            deflated_value = property_obj.deflate(value)
+            # handle special operators
+            if operator == _SPECIAL_OPERATOR_IN:
+                if not isinstance(value, tuple) and not isinstance(value, list):
+                    raise ValueError('Value must be a tuple or list for IN operation {}={}'.format(key, value))
+                deflated_value = [property_obj.deflate(v) for v in value]
+            elif operator == _SPECIAL_OPERATOR_ISNULL:
+                if not isinstance(value, bool):
+                    raise ValueError('Value must be a bool for isnull operation on {}'.format(key))
+                operator = 'IS NULL' if value else 'IS NOT NULL'
+                deflated_value = None
+            elif operator in _REGEX_OPERATOR_TABLE.values():
+                deflated_value = property_obj.deflate(value)
+                if not isinstance(deflated_value, basestring):
+                    raise ValueError('Must be a string value for {}'.format(key))
+                if operator in _STRING_REGEX_OPERATOR_TABLE.values():
+                    deflated_value = re.escape(deflated_value)
+                deflated_value = operator.format(deflated_value)
+                operator = _SPECIAL_OPERATOR_REGEX
+            else:
+                deflated_value = property_obj.deflate(value)
 
-        output[prop] = (operator, deflated_value)
+        # map property to correct property name in the database
+        db_property = cls.defined_properties(rels=False)[prop].db_property or prop
+
+        output[db_property] = (operator, deflated_value)
 
     return output
 
@@ -103,136 +205,6 @@ def process_has_args(cls, kwargs):
     return match, dont_match
 
 
-class BaseSet(object):
-    def all(self):
-        return QueryBuilder(self).build_ast()._execute()
-
-    def __iter__(self):
-        return (i for i in QueryBuilder(self).build_ast()._execute())
-
-    def __len__(self):
-        return QueryBuilder(self).build_ast()._count()
-
-    def __bool__(self):
-        return QueryBuilder(self).build_ast()._count() > 0
-
-    def __nonzero__(self):
-        return QueryBuilder(self).build_ast()._count() > 0
-
-    def __contains__(self, obj):
-        if isinstance(obj, StructuredNode):
-            if hasattr(obj, '_id'):
-                return QueryBuilder(self).build_ast()._contains(int(obj._id))
-            raise ValueError("Unsaved node: " + repr(obj))
-        else:
-            raise ValueError("Expecting StructuredNode instance")
-
-    def __getitem__(self, key):
-        if isinstance(key, slice):
-            if key.stop and key.start:
-                self.limit = key.stop - key.start
-                self.skip = key.start
-            elif key.stop:
-                self.limit = key.stop
-            elif key.start:
-                self.skip = key.start
-        elif isinstance(key, int):
-            self.skip = key
-            self.limit = 1
-
-        return QueryBuilder(self).build_ast()._execute()
-
-
-class NodeSet(BaseSet):
-    """
-    a set of matched nodes of a single type
-        source: how to produce the set of nodes
-        node_cls: what type of nodes are they
-    """
-    def __init__(self, source):
-        self.source = source  # could be a Traverse object or a node class
-        if isinstance(source, Traversal):
-            self.source_class = source.target_class
-        elif inspect.isclass(source) and issubclass(source, StructuredNode):
-            self.source_class = source
-        elif isinstance(source, StructuredNode):
-            self.source_class = source.__class__
-        else:
-            raise ValueError("Bad source for nodeset " + repr(source))
-
-        # setup Traversal objects using relationship definitions
-        install_traversals(self.source_class, self)
-
-        self.filters = []
-
-        # used by has()
-        self.must_match = {}
-        self.dont_match = {}
-
-    def get(self, **kwargs):
-        output = process_filter_args(self.source_class, kwargs)
-        self.filters.append(output)
-        self.limit = 2
-        result = QueryBuilder(self).build_ast()._execute()
-        if len(result) > 1:
-            raise MultipleNodesReturned(repr(kwargs))
-        elif not result:
-            raise self.source_class.DoesNotExist(repr(kwargs))
-        else:
-            return result[0]
-
-    def filter(self, **kwargs):
-        output = process_filter_args(self.source_class, kwargs)
-        self.filters.append(output)
-        return self
-
-    def exclude(self, **kwargs):
-        output = process_filter_args(self.source_class, kwargs)
-        self.filters.append({'__NOT__': output})
-        return self
-
-    def has(self, **kwargs):
-        must_match, dont_match = process_has_args(self.source_class, kwargs)
-        self.must_match.update(must_match)
-        self.dont_match.update(dont_match)
-        return self
-
-
-class Traversal(BaseSet):
-    """
-        source: start of traversal could be any of: StructuredNode instance, StucturedNode class, NodeSet
-        definition: relationship definition
-    """
-    def __init__(self, source, key, definition):
-        self.source = source
-
-        if isinstance(source, Traversal):
-            self.source_class = source.target_class
-        elif inspect.isclass(source) and issubclass(source, StructuredNode):
-            self.source_class = source
-        elif isinstance(source, StructuredNode):
-            self.source_class = source.__class__
-        elif isinstance(source, NodeSet):
-            self.source_class = source.source_class
-        else:
-            raise ValueError("Bad source for traversal: {}".format(repr(source)))
-
-        self.definition = definition
-        self.target_class = definition['node_class']
-        self.name = key
-        self.filters = []
-
-    def match(self, **kwargs):
-        if 'model' not in self.definition:
-            raise ValueError("match() only available on relationships with a model")
-        if kwargs:
-            self.filters.append(process_filter_args(self.definition['model'], kwargs))
-        return self
-
-    def _in_node_set(self):
-        return NodeSet(self)
-
-
 class QueryBuilder(object):
     def __init__(self, node_set):
         self.node_set = node_set
@@ -262,6 +234,9 @@ class QueryBuilder(object):
 
             self.build_additional_match(ident, source)
 
+            if hasattr(source, '_order_by'):
+                self.build_order_by(ident, source)
+
             if source.filters:
                 self.build_where_stmt(ident, source.filters)
 
@@ -275,6 +250,14 @@ class QueryBuilder(object):
         self._ident_count += 1
         return 'r' + str(self._ident_count)
 
+    def build_order_by(self, ident, source):
+        if '?' in source._order_by:
+            self._ast['with'] = '{}, rand() as r'.format(ident)
+            self._ast['order_by'] = 'r'
+        else:
+            self._ast['order_by'] = ['{}.{}'.format(ident, p)
+                                     for p in source._order_by]
+
     def build_traversal(self, traversal):
         """
         traverse a relationship from a node to a set of nodes
@@ -289,7 +272,7 @@ class QueryBuilder(object):
         self._ast['result_class'] = traversal.target_class
 
         rel_ident = self.create_ident()
-        stmt = rel_helper(lhs=lhs_ident, rhs=rhs_ident, ident=rel_ident, **traversal.definition)
+        stmt = _rel_helper(lhs=lhs_ident, rhs=rhs_ident, ident=rel_ident, **traversal.definition)
         self._ast['match'].append(stmt)
 
         if traversal.filters:
@@ -304,7 +287,7 @@ class QueryBuilder(object):
 
         place_holder = self._register_place_holder(ident)
         self._ast['start'].append('{} = node({{{}}})'.format(ident, place_holder))
-        self._query_params[place_holder] = node._id
+        self._query_params[place_holder] = node.id
 
         self._ast['return'] = ident
         self._ast['result_class'] = node.__class__
@@ -324,25 +307,23 @@ class QueryBuilder(object):
         """
             handle additional matches supplied by 'has()' calls
         """
-        # TODO add support for labels
         source_ident = ident
 
         for key, value in node_set.must_match.items():
-            label = ':' + value['node_class'].__label__
             if isinstance(value, dict):
-                stmt = rel_helper(lhs=source_ident, rhs=label, ident='', **value)
+                label = ':' + value['node_class'].__label__
+                stmt = _rel_helper(lhs=source_ident, rhs=label, ident='', **value)
                 self._ast['where'].append(stmt)
-            elif isinstance(value, tuple):
-                rel_manager, ns = value
-                self.add_node_set(ns, key)
+            else:
+                raise ValueError("Expecting dict got: " + repr(value))
 
         for key, val in node_set.dont_match.items():
-            label = ':' + val['node_class'].__label__
             if isinstance(val, dict):
-                stmt = rel_helper(lhs=source_ident, rhs=label, ident='', **val)
+                label = ':' + val['node_class'].__label__
+                stmt = _rel_helper(lhs=source_ident, rhs=label, ident='', **val)
                 self._ast['where'].append('NOT ' + stmt)
             else:
-                raise ValueError("WTF? " + repr(val))
+                raise ValueError("Expecting dict got: " + repr(val))
 
     def _register_place_holder(self, key):
         if key in self._place_holder_registry:
@@ -366,11 +347,14 @@ class QueryBuilder(object):
 
             for prop, op_and_val in row.items():
                 op, val = op_and_val
-                place_holder = self._register_place_holder(ident + '_' + prop)
-                statement = '{} {}.{} {} {{{}}}'.format(
-                    'NOT' if negate else '', ident, prop, op, place_holder)
+                if op in _UNARY_OPERATORS:
+                    # unary operators do not have a parameter
+                    statement = '{} {}.{} {}'.format('NOT' if negate else '', ident, prop, op)
+                else:
+                    place_holder = self._register_place_holder(ident + '_' + prop)
+                    statement = '{} {}.{} {} {{{}}}'.format('NOT' if negate else '', ident, prop, op, place_holder)
+                    self._query_params[place_holder] = val
                 stmts.append(statement)
-                self._query_params[place_holder] = val
 
         self._ast['where'].append(' AND '.join(stmts))
 
@@ -382,14 +366,22 @@ class QueryBuilder(object):
             query += ', '.join(self._ast['start'])
 
         query += ' MATCH '
-        query += ', '.join(self._ast['match'])
+        query += ', '.join(['({})'.format(i) for i in self._ast['match']])
 
         if 'where' in self._ast and self._ast['where']:
             query += ' WHERE '
             query += ' AND '.join(self._ast['where'])
 
+        if 'with' in self._ast and self._ast['with']:
+            query += ' WITH '
+            query += self._ast['with']
+
         query += ' RETURN ' + self._ast['return']
 
+        if 'order_by' in self._ast and self._ast['order_by']:
+            query += ' ORDER BY '
+            query += ', '.join(self._ast['order_by'])
+
         if 'skip' in self._ast:
             query += ' SKIP {0:d}'.format(self._ast['skip'])
 
@@ -400,6 +392,8 @@ class QueryBuilder(object):
 
     def _count(self):
         self._ast['return'] = 'count({})'.format(self._ast['return'])
+        # drop order_by, results in an invalid query
+        self._ast.pop('order_by', None)
         query = self.build_query()
         results, _ = db.cypher_query(query, self._query_params)
         return int(results[0][0])
@@ -418,3 +412,250 @@ class QueryBuilder(object):
         if results:
             return [self._ast['result_class'].inflate(n[0]) for n in results]
         return []
+
+
+class BaseSet(object):
+    """
+    Base class for all node sets.
+
+    Contains common python magic methods, __len__, __contains__ etc
+    """
+    query_cls = QueryBuilder
+
+    def all(self):
+        """
+        Return all nodes belonging to the set
+        :return: list of nodes
+        :rtype: list
+        """
+        return self.query_cls(self).build_ast()._execute()
+
+    def __iter__(self):
+        return (i for i in self.query_cls(self).build_ast()._execute())
+
+    def __len__(self):
+        return self.query_cls(self).build_ast()._count()
+
+    def __bool__(self):
+        return self.query_cls(self).build_ast()._count() > 0
+
+    def __nonzero__(self):
+        return self.query_cls(self).build_ast()._count() > 0
+
+    def __contains__(self, obj):
+        if isinstance(obj, StructuredNode):
+            if hasattr(obj, 'id'):
+                return self.query_cls(self).build_ast()._contains(int(obj.id))
+            raise ValueError("Unsaved node: " + repr(obj))
+        else:
+            raise ValueError("Expecting StructuredNode instance")
+
+    def __getitem__(self, key):
+        if isinstance(key, slice):
+            if key.stop and key.start:
+                self.limit = key.stop - key.start
+                self.skip = key.start
+            elif key.stop:
+                self.limit = key.stop
+            elif key.start:
+                self.skip = key.start
+
+            return self.query_cls(self).build_ast()._execute()
+
+        elif isinstance(key, int):
+            self.skip = key
+            self.limit = 1
+
+            return self.query_cls(self).build_ast()._execute()[0]
+
+
+class NodeSet(BaseSet):
+    """
+    A class representing as set of nodes matching common query parameters
+    """
+    def __init__(self, source):
+        self.source = source  # could be a Traverse object or a node class
+        if isinstance(source, Traversal):
+            self.source_class = source.target_class
+        elif inspect.isclass(source) and issubclass(source, StructuredNode):
+            self.source_class = source
+        elif isinstance(source, StructuredNode):
+            self.source_class = source.__class__
+        else:
+            raise ValueError("Bad source for nodeset " + repr(source))
+
+        # setup Traversal objects using relationship definitions
+        install_traversals(self.source_class, self)
+
+        self.filters = []
+
+        # used by has()
+        self.must_match = {}
+        self.dont_match = {}
+
+    def get(self, **kwargs):
+        """
+        Retrieve one node from the set matching supplied parameters
+
+        :param kwargs: same syntax as `filter()`
+        :return: node
+        """
+        output = process_filter_args(self.source_class, kwargs)
+        if output:
+            self.filters.append(output)
+        self.limit = 2
+        result = self.query_cls(self).build_ast()._execute()
+        if len(result) > 1:
+            raise MultipleNodesReturned(repr(kwargs))
+        elif not result:
+            raise self.source_class.DoesNotExist(repr(kwargs))
+        else:
+            return result[0]
+
+    def get_or_none(self, **kwargs):
+        """
+        Retrieve a node from the set matching supplied parameters or return none
+
+        :param kwargs: same syntax as `filter()`
+        :return: node or none
+        """
+        try:
+            return self.get(**kwargs)
+        except self.source_class.DoesNotExist:
+            pass
+
+    def filter(self, **kwargs):
+        """
+        Apply filters to the existing nodes in the set.
+
+        :param kwargs: filter parameters
+
+            Filters mimic Django's syntax with the double '__' to separate field and operators.
+
+            e.g `.filter(salary__gt=20000)` results in `salary > 20000`.
+
+            The following operators are available:
+
+             * 'lt': less than
+             * 'gt': greater than
+             * 'lte': less than or equal to
+             * 'gte': greater than or equal to
+             * 'ne': not equal to
+             * 'in': matches one of list (or tuple)
+             * 'isnull': is null
+             * 'regex': matches supplied regex (neo4j regex format)
+             * 'exact': exactly match string (just '=')
+             * 'iexact': case insensitive match string
+             * 'contains': contains string
+             * 'icontains': case insensitive contains
+             * 'startswith': string starts with
+             * 'istartswith': case insensitive string starts with
+             * 'endswith': string ends with
+             * 'iendswith': case insensitive string ends with
+
+        :return: self
+        """
+
+        output = process_filter_args(self.source_class, kwargs)
+        if output:
+            self.filters.append(output)
+        return self
+
+    def exclude(self, **kwargs):
+        """
+        Exclude nodes from the NodeSet via filters.
+
+        :param kwargs: filter parameters see syntax for the filter method
+        :return: self
+        """
+        output = process_filter_args(self.source_class, kwargs)
+        if output:
+            self.filters.append({'__NOT__': output})
+        return self
+
+    def has(self, **kwargs):
+        must_match, dont_match = process_has_args(self.source_class, kwargs)
+        self.must_match.update(must_match)
+        self.dont_match.update(dont_match)
+        return self
+
+    def order_by(self, *props):
+        """
+        Order by properties. Prepend with minus to do descending. Pass None to
+        remove ordering.
+        """
+        should_remove = len(props) == 1 and props[0] is None
+        if not hasattr(self, '_order_by') or should_remove:
+            self._order_by = []
+            if should_remove:
+                return self
+        if '?' in props:
+            self._order_by.append('?')
+        else:
+            for prop in props:
+                prop = prop.strip()
+                if prop.startswith('-'):
+                    prop = prop[1:]
+                    desc = True
+                else:
+                    desc = False
+
+                if prop not in self.source_class.defined_properties(rels=False):
+                    raise ValueError("No such property {} on {}".format(
+                        prop, self.source_class.__name__))
+
+                property_obj = getattr(self.source_class, prop)
+                if isinstance(property_obj, AliasProperty):
+                    prop = property_obj.aliased_to()
+
+                self._order_by.append(prop + (' DESC' if desc else ''))
+
+        return self
+
+
+class Traversal(BaseSet):
+    """
+    Models a traversal from a node to another, inherits from BaseSet
+    """
+
+    def __init__(self, source, key, definition):
+        """
+        Create a traversal
+        :param source: start of traversal could be any of: StructuredNode instance, StucturedNode class, NodeSet
+        :param key:
+        :param definition: relationship definition
+        """
+        self.source = source
+
+        if isinstance(source, Traversal):
+            self.source_class = source.target_class
+        elif inspect.isclass(source) and issubclass(source, StructuredNode):
+            self.source_class = source
+        elif isinstance(source, StructuredNode):
+            self.source_class = source.__class__
+        elif isinstance(source, NodeSet):
+            self.source_class = source.source_class
+        else:
+            raise ValueError("Bad source for traversal: {}".format(repr(source)))
+
+        self.definition = definition
+        self.target_class = definition['node_class']
+        self.name = key
+        self.filters = []
+
+    def match(self, **kwargs):
+        """
+        Traverse relationships with properties matching the given parameters.
+
+            e.g: `.match(price__lt=10)`
+
+        :param kwargs: see `NodeSet.filter()` for syntax
+        :return: self
+        """
+        if 'model' not in self.definition:
+            raise ValueError("match() only available on relationships with a model")
+        if kwargs:
+            output = process_filter_args(self.definition['model'], kwargs) 
+            if output:
+                self.filters.append(output)
+        return self
