@@ -1,3 +1,4 @@
+from __future__ import print_function
 from neomodel import (StructuredNode, StringProperty, IntegerProperty, ArrayProperty)
 from neomodel.exception import RequiredProperty, UniqueProperty
 
@@ -5,7 +6,6 @@ from neomodel.exception import RequiredProperty, UniqueProperty
 class User(StructuredNode):
     email = StringProperty(unique_index=True, required=True)
     age = IntegerProperty(index=True)
-    aliases = ArrayProperty()
 
     @property
     def email_alias(self):
@@ -16,13 +16,18 @@ class User(StructuredNode):
         self.email = value
 
 
-def test_array_properties():
-    user = User(email='foo@bar.com', aliases=['Tim', 'Bob']).save()
-    assert user
-    assert 'Tim' in user.aliases
-    user = User.nodes.get(email='foo@bar.com')
-    assert user
-    assert 'Tim' in user.aliases
+def test_issue_233():
+    class BaseIssue233(StructuredNode):
+        __abstract_node__ = True
+
+        def __getitem__(self, item):
+            return self.__dict__[item]
+
+    class Issue233(BaseIssue233):
+        uid = StringProperty(unique_index=True, required=True)
+
+    i = Issue233(uid='testgetitem').save()
+    assert i['uid'] == 'testgetitem'
 
 
 def test_issue_72():
@@ -39,18 +44,31 @@ def test_required():
         assert False
 
 
-def test_get():
+def test_repr_and_str():
+    u = User(email='robin@test.com', age=3)
+    print(repr(u))
+    print(str(u))
+    assert True
+
+
+def test_get_and_get_or_none():
     u = User(email='robin@test.com', age=3)
     assert u.save()
     rob = User.nodes.get(email='robin@test.com')
     assert rob.email == 'robin@test.com'
     assert rob.age == 3
 
+    rob = User.nodes.get_or_none(email='robin@test.com')
+    assert rob.email == 'robin@test.com'
+
+    n = User.nodes.get_or_none(email='robin@nothere.com')
+    assert n is None
+
 
 def test_save_to_model():
     u = User(email='jim@test.com', age=3)
     assert u.save()
-    assert u._id > 0
+    assert u.id > 0
     assert u.email == 'jim@test.com'
     assert u.age == 3
 
@@ -147,3 +165,61 @@ def test_refresh():
     c.refresh()
     assert c.age == 20
     assert c.my_custom_prop == 'value'
+
+
+def test_setting_value_to_none():
+    c = Customer2(email='alice@bob.com', age=42).save()
+    assert c.age is not None
+
+    c.age = None
+    c.save()
+
+    copy = Customer2.nodes.get(email='alice@bob.com')
+    assert copy.age is None
+
+
+def test_inheritance():
+    class User(StructuredNode):
+        __abstract_node__ = True
+        name = StringProperty(unique_index=True)
+
+    class Shopper(User):
+        balance = IntegerProperty(index=True)
+
+        def credit_account(self, amount):
+            self.balance = self.balance + int(amount)
+            self.save()
+
+    jim = Shopper(name='jimmy', balance=300).save()
+    jim.credit_account(50)
+
+    assert Shopper.__label__ == 'Shopper'
+    assert jim.balance == 350
+    assert len(jim.inherited_labels()) == 1
+    assert len(jim.labels()) == 1
+    assert jim.labels()[0] == 'Shopper'
+
+
+def test_mixins():
+    class UserMixin(object):
+        name = StringProperty(unique_index=True)
+        password = StringProperty()
+
+    class CreditMixin(object):
+        balance = IntegerProperty(index=True)
+
+        def credit_account(self, amount):
+            self.balance = self.balance + int(amount)
+            self.save()
+
+    class Shopper2(StructuredNode, UserMixin, CreditMixin):
+        pass
+
+    jim = Shopper2(name='jimmy', balance=300).save()
+    jim.credit_account(50)
+
+    assert Shopper2.__label__ == 'Shopper2'
+    assert jim.balance == 350
+    assert len(jim.inherited_labels()) == 1
+    assert len(jim.labels()) == 1
+    assert jim.labels()[0] == 'Shopper2'
