@@ -1,12 +1,12 @@
 from neomodel import (StructuredNode, RelationshipTo, RelationshipFrom,
-        Relationship, StringProperty, IntegerProperty, One)
+        Relationship, StringProperty, IntegerProperty, StructuredRel, One)
 
 
 class Person(StructuredNode):
     name = StringProperty(unique_index=True)
     age = IntegerProperty(index=True)
     is_from = RelationshipTo('Country', 'IS_FROM')
-    knows = Relationship('Person', 'KNOWS')
+    knows = Relationship(u'Person', 'KNOWS')  # use unicode to check issue
 
     @property
     def special_name(self):
@@ -92,13 +92,20 @@ def test_either_direction_connect():
     assert sakis.knows.is_connected(rey)
     sakis.knows.connect(rey)
 
-    result, meta = sakis.cypher("""START us=node({self}), them=node({them})
+    result, meta = sakis.cypher("""MATCH (us), (them)
+            WHERE id(us)={self} and id(them)={them}
             MATCH (us)-[r:KNOWS]-(them) RETURN COUNT(r)""",
-            {'them': rey._id})
+            {'them': rey.id})
     assert int(result[0][0]) == 1
 
+    rel = rey.knows.relationship(sakis)
+    assert isinstance(rel, StructuredRel)
 
-def test_search():
+    rels = rey.knows.all_relationships(sakis)
+    assert isinstance(rels[0], StructuredRel)
+
+
+def test_search_and_filter_and_exclude():
     fred = Person(name='Fred', age=13).save()
     zz = Country(code='ZZ').save()
     zx = Country(code='ZX').save()
@@ -109,6 +116,12 @@ def test_search():
     result = fred.is_from.search(code='ZX')
     assert result[0].code == 'ZX'
 
+    result = fred.is_from.filter(code='ZX')
+    assert result[0].code == 'ZX'
+
+    result = fred.is_from.exclude(code='ZZ').exclude(code='ZY')
+    assert result[0].code == 'ZX' and len(result) == 1
+
 
 def test_custom_methods():
     u = Person(name='Joe90', age=13).save()
@@ -156,3 +169,4 @@ def test_props_relationship():
         assert True
     else:
         assert False
+
