@@ -1,4 +1,5 @@
 from neomodel import db, StructuredNode, StringProperty, UniqueProperty
+from neo4j.addressing import AddressError
 
 
 class Person(StructuredNode):
@@ -62,3 +63,29 @@ def test_transaction_as_a_context():
         assert True
     else:
         assert False
+
+
+def test_query_inside_transaction():
+    for p in Person.nodes:
+        p.delete()
+
+    with db.transaction:
+        Person(name='Alice').save()
+        Person(name='Bob').save()
+
+        assert len([p.name for p in Person.nodes]) == 2
+
+
+def test_set_connection_works():
+    assert Person(name='New guy').save()
+    from socket import gaierror
+
+    old_url = db.url
+    try:
+        db.set_connection('bolt://user:password@nowhere:7687')
+        assert False  # Shouldnt get here
+    except AddressError:
+        assert True
+        db.set_connection(old_url)
+    # set connection back
+    assert Person(name='New guy2').save()
\ No newline at end of file
